@@ -1,7 +1,8 @@
-// Copyright (c) 2023 Sophie Herold
+// Copyright (c) 2023-2024 Sophie Herold
 // Copyright (c) 2023 Alice Mikhaylenko
 // Copyright (c) 2023 Christopher Davis
 // Copyright (c) 2023 Lubosz Sarnecki
+// Copyright (c) 2024 Fina Wilke
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,44 +23,103 @@
 //!
 //! This widget it similar to `AdwCarousel`.
 
-use crate::deps::*;
-use crate::widgets::LpImagePage;
+use std::cell::{Cell, OnceCell, RefCell};
 
 use adw::prelude::*;
 use adw::subclass::prelude::*;
+use glib::Properties;
 use indexmap::IndexMap;
+use log::error;
 use once_cell::sync::Lazy;
 
-use std::cell::{Cell, OnceCell, RefCell};
+use crate::deps::*;
+use crate::widgets::LpImagePage;
 
-static SCROLL_DAMPING_RATIO: f64 = 1.;
-static SCROLL_MASS: f64 = 0.5;
-static SCROLL_STIFFNESS: f64 = 500.;
+const SCROLL_DAMPING_RATIO: f64 = 1.;
+const SCROLL_MASS: f64 = 0.5;
+const SCROLL_STIFFNESS: f64 = 500.;
+
+/// Duration for cards animation
+const STEP_DURATION: u32 = 250;
+/// Offset of card at the beginning in app pixels
+const STEP_PIXEL_SHIFT: f32 = 100.;
+/// Progress point at which the move-out animation starts.
+/// The progress is a value between 0.0 and 1.0
+const STEP_MOVE_OUT_DELAY: f32 = 0.5;
 
 /// Space between images in application pixels
 ///
 /// This is combined with the percent component
-static PAGE_SPACING_FIXED: f32 = 25.;
+const PAGE_SPACING_FIXED: f32 = 25.;
 
 /// Space between images as factor of width
 const PAGE_SPACING_PERCENT: f32 = 0.02;
 
+#[derive(Debug)]
+enum PositionTracking {
+    /// Animatable position, 0.0 first image, 1.0 second image etc
+    Position(f64),
+    StackedCards(StackedCards),
+}
+
+impl Default for PositionTracking {
+    fn default() -> Self {
+        PositionTracking::Position(0.)
+    }
+}
+
+impl PositionTracking {
+    fn set_position(&mut self, position: f64) {
+        *self = Self::Position(position);
+    }
+
+    fn set_progress(&mut self, progress: f64) {
+        match self {
+            Self::Position(_) => {
+                log::error!("Trying to set StackedCards progress while not in animation state.")
+            }
+            Self::StackedCards(cards) => cards.progress = progress,
+        };
+    }
+}
+
+#[derive(Debug)]
+struct StackedCards {
+    prev_image: LpImagePage,
+    progress: f64,
+    backward: bool,
+}
+
+impl StackedCards {
+    fn new(prev_image: LpImagePage, backward: bool) -> Self {
+        Self {
+            prev_image,
+            progress: 0.,
+            backward,
+        }
+    }
+}
+
 mod imp {
-    use super::*;
     use glib::subclass::Signal;
 
-    #[derive(Default)]
+    use super::*;
+
+    #[derive(Default, Properties)]
+    #[properties(wrapper_type = super::LpSlidingView)]
     pub struct LpSlidingView {
         /// Pages that can be slided through
         pub(super) pages: RefCell<Vec<LpImagePage>>,
         /// Page that is currently shown or the animation is animating towards
+        #[property(get)]
         pub(super) current_page: RefCell<Option<LpImagePage>>,
         /// Animatable position, 0.0 first image, 1.0 second image etc
-        pub(super) position: Cell<f64>,
+        pub(super) position_tracking: RefCell<PositionTracking>,
         /// Move position to not break animations when pages are removed/added
         pub(super) position_shift: Cell<f64>,
         /// The animation used to animate image changes
         pub(super) scroll_animation: OnceCell<adw::SpringAnimation>,
+        pub(super) step_animation: OnceCell<adw::TimedAnimation>,
         /// Implements swiping
         pub(super) swipe_tracker: OnceCell<adw::SwipeTracker>,
     }
@@ -74,23 +134,11 @@ mod imp {
 
     impl ObjectImpl for LpSlidingView {
         fn properties() -> &'static [glib::ParamSpec] {
-            static PROPERTIES: Lazy<Vec<glib::ParamSpec>> = Lazy::new(|| {
-                vec![
-                    glib::ParamSpecObject::builder::<LpImagePage>("current-page")
-                        .read_only()
-                        .build(),
-                ]
-            });
-
-            PROPERTIES.as_ref()
+            Self::derived_properties()
         }
 
-        fn property(&self, _id: usize, pspec: &glib::ParamSpec) -> glib::Value {
-            let obj = self.obj();
-            match pspec.name() {
-                "current-page" => obj.current_page().to_value(),
-                name => unimplemented!("property {name}"),
-            }
+        fn property(&self, id: usize, pspec: &glib::ParamSpec) -> glib::Value {
+            self.derived_property(id, pspec)
         }
 
         fn signals() -> &'static [Signal] {
@@ -112,19 +160,32 @@ mod imp {
                 .upper_overshoot(true)
                 .build();
 
-            swipe_tracker.connect_begin_swipe(
-                glib::clone!(@weak obj => move |_| obj.scroll_animation().pause()),
-            );
-
-            swipe_tracker.connect_update_swipe(glib::clone!(@weak obj => move |_, position| {
-                obj.set_position(position);
-            }));
-
-            swipe_tracker.connect_end_swipe(glib::clone!(@weak obj => move |_, velocity, to| {
-                if let Some(page) = obj.page_at(to) {
-                    obj.scroll_to_velocity(&page, velocity);
+            swipe_tracker.connect_begin_swipe(glib::clone!(
+                #[weak]
+                obj,
+                move |_| {
+                    obj.scroll_animation().pause();
+                    obj.step_animation().pause();
                 }
-            }));
+            ));
+
+            swipe_tracker.connect_update_swipe(glib::clone!(
+                #[weak]
+                obj,
+                move |_, position| {
+                    obj.set_position(position);
+                }
+            ));
+
+            swipe_tracker.connect_end_swipe(glib::clone!(
+                #[weak]
+                obj,
+                move |_, velocity, to| {
+                    if let Some(page) = obj.page_at(to) {
+                        obj.scroll_to(&page, velocity);
+                    }
+                }
+            ));
 
             self.swipe_tracker.set(swipe_tracker).unwrap();
 
@@ -132,8 +193,12 @@ mod imp {
             let scroll_controller =
                 gtk::EventControllerScroll::new(gtk::EventControllerScrollFlags::HORIZONTAL);
 
-            scroll_controller.connect_scroll(
-                glib::clone!(@weak obj => @default-return glib::Propagation::Proceed, move |_, x, _| {
+            scroll_controller.connect_scroll(glib::clone!(
+                #[weak]
+                obj,
+                #[upgrade_or]
+                glib::Propagation::Proceed,
+                move |_, x, _| {
                     let direction_sign = if obj.imp().is_rtl() { -1. } else { 1. };
 
                     if x * direction_sign > 0. {
@@ -159,8 +224,8 @@ mod imp {
                             glib::Propagation::Stop
                         }
                     }
-                }),
-            );
+                }
+            ));
 
             obj.add_controller(scroll_controller);
         }
@@ -175,30 +240,39 @@ mod imp {
 
     impl WidgetImpl for LpSlidingView {
         fn size_allocate(&self, width: i32, height: i32, _baseline: i32) {
-            let scroll_position = self.position.get() as f32;
-            let position_shift = self.position_shift.get() as f32;
+            let obj = self.obj();
+
+            // Unparent deleted children if not in animation that might contain them
+            if matches!(
+                *self.position_tracking.borrow(),
+                PositionTracking::Position(_)
+            ) {
+                if let Some(mut child) = obj.first_child() {
+                    let mut children = vec![child.clone()];
+                    while let Some(new_child) = child.next_sibling() {
+                        child = new_child;
+                        children.push(child.clone());
+                    }
+
+                    for child in children {
+                        if let Ok(page) = child.downcast::<LpImagePage>() {
+                            if obj.index_of(&page).is_none() {
+                                page.unparent();
+                            }
+                        }
+                    }
+                }
+            }
 
             for (page_index, page) in self.pages.borrow().iter().enumerate() {
-                let page_position = page_index as f32;
-
-                // reverse page order for RTL languages
-                let direction_sign = if self.is_rtl() { -1. } else { 1. };
-
-                // This positions the pages within the carousel and shifts them
-                // according to the position that should currently be shown.
-                let x = direction_sign
-                    * (page_position - scroll_position - position_shift)
-                    * (width as f32 + self.page_spacing(width));
-
-                // Only show visible images
-                if page_position == (scroll_position + position_shift).floor()
-                    || page_position == (scroll_position + position_shift).ceil()
-                {
-                    page.set_child_visible(true);
-                    let transform = gsk::Transform::new().translate(&graphene::Point::new(x, 0.));
-                    page.allocate(width, height, 0, Some(transform));
-                } else {
-                    page.set_child_visible(false);
+                match *self.position_tracking.borrow() {
+                    PositionTracking::Position(_) => {
+                        self.size_allocate_position(page_index, page, width, height);
+                    }
+                    PositionTracking::StackedCards(_) => {
+                        // Hide all and show to relevant widgets later
+                        page.set_child_visible(false);
+                    }
                 }
 
                 if !page.scrolled_window().is_mapped() {
@@ -206,6 +280,11 @@ mod imp {
                     page.scrolled_window().allocate(width, height, 0, None);
                 }
             }
+
+            if let PositionTracking::StackedCards(stacked_cards) = &*self.position_tracking.borrow()
+            {
+                self.size_allocate_stacked_cards(stacked_cards, width, height);
+            }
         }
 
         fn measure(&self, orientation: gtk::Orientation, for_size: i32) -> (i32, i32, i32, i32) {
@@ -239,9 +318,7 @@ mod imp {
 
         fn distance(&self) -> f64 {
             let obj = self.obj();
-            // TODO: Fix deprecation
-            #[allow(deprecated)]
-            let width = obj.allocation().width();
+            let width = obj.width();
 
             width as f64 + self.page_spacing(width) as f64
         }
@@ -267,6 +344,94 @@ mod imp {
     }
 
     impl LpSlidingView {
+        /// Usualy allocate or during scroll animation
+        fn size_allocate_position(
+            &self,
+            page_index: usize,
+            page: &LpImagePage,
+            width: i32,
+            height: i32,
+        ) {
+            let scroll_position = self.obj().position() as f32;
+            let position_shift = self.position_shift.get() as f32;
+            let page_position = page_index as f32;
+
+            // Reset from potential stacked cards animation
+            page.set_opacity(1.);
+
+            // reverse page order for RTL languages
+            let direction_sign = if self.is_rtl() { -1. } else { 1. };
+
+            // This positions the pages within the carousel and shifts them
+            // according to the position that should currently be shown.
+            let x = direction_sign
+                * (page_position - scroll_position - position_shift)
+                * (width as f32 + self.page_spacing(width));
+
+            // Only show visible images
+            if page_position == (scroll_position + position_shift).floor()
+                || page_position == (scroll_position + position_shift).ceil()
+            {
+                page.set_child_visible(true);
+                let transform = gsk::Transform::new().translate(&graphene::Point::new(x, 0.));
+                page.allocate(width, height, 0, Some(transform));
+            } else {
+                page.set_child_visible(false);
+            }
+        }
+
+        /// Allocate during stacked card animation
+        fn size_allocate_stacked_cards(
+            &self,
+            stacked_cards: &StackedCards,
+            width: i32,
+            height: i32,
+        ) {
+            let obj = self.obj().to_owned();
+            if let Some(new_page) = obj.current_page() {
+                let prev_page = &stacked_cards.prev_image;
+
+                let mut direction_sign = if stacked_cards.backward { -1. } else { 1. };
+                if self.is_rtl() {
+                    direction_sign *= -1.;
+                }
+
+                let progress = stacked_cards.progress;
+
+                //New Page
+
+                // Ensure that new image is on top
+                new_page.unparent();
+                new_page.insert_after(&obj, Some(prev_page));
+
+                // Don't show spinner or error during animation
+                if new_page.image().is_loaded() {
+                    new_page.set_child_visible(true);
+                } else {
+                    // Set explicity since re-parenting resets the state
+                    new_page.set_child_visible(false);
+                }
+
+                let x = direction_sign * STEP_PIXEL_SHIFT * (1. - progress as f32);
+                let transform = gsk::Transform::new().translate(&graphene::Point::new(x, 0.));
+                new_page.allocate(width, height, 0, Some(transform));
+                new_page.set_opacity(progress);
+
+                // Prev Page
+                if prev_page.image().is_loaded() {
+                    prev_page.set_child_visible(true);
+                }
+
+                let x = -direction_sign
+                    * f32::max(
+                        0.,
+                        (progress as f32 - STEP_MOVE_OUT_DELAY) * STEP_PIXEL_SHIFT,
+                    );
+                let transform = gsk::Transform::new().translate(&graphene::Point::new(x, 0.));
+                prev_page.allocate(width, height, 0, Some(transform));
+            }
+        }
+
         fn page_spacing(&self, width: i32) -> f32 {
             width as f32 * PAGE_SPACING_PERCENT + PAGE_SPACING_FIXED
         }
@@ -285,6 +450,14 @@ glib::wrapper! {
 }
 
 impl LpSlidingView {
+    /// Returns a struct that allows lazy updates
+    ///
+    /// Lazy updates don't apply their notifications directly but only when the
+    /// editor struct is dropped.
+    pub fn editor(&self) -> SlidingViewEditor {
+        SlidingViewEditor::new(self.clone())
+    }
+
     /// Add page to the end
     pub fn append(&self, page: &LpImagePage) {
         self.insert(page, self.n_pages());
@@ -331,19 +504,23 @@ impl LpSlidingView {
     }
 
     /// Removes the page
-    pub fn remove(&self, page: &LpImagePage) {
+    fn remove(&self, page: &LpImagePage, lazy: bool) {
         let current_index = self.current_index();
 
         if let Some(index) = self.index_of(page) {
+            // Only unparent if far enough away to not be involved in delete animation
+            if current_index.map_or(false, |x| x.abs_diff(index) as isize > 1) {
+                page.unparent();
+            }
+
             self.imp().pages.borrow_mut().remove(index);
-            page.unparent();
 
             self.queue_allocate();
 
             self.shift_position(current_index);
 
-            if self.is_empty() {
-                self.clear();
+            if self.is_empty() && !lazy {
+                self.clear(false);
             }
         } else {
             log::error!("Trying to remove non-existent page.");
@@ -358,9 +535,14 @@ impl LpSlidingView {
         });
     }
 
+    fn emit_target_page_reached(&self) {
+        self.emit_by_name::<()>("target-page-reached", &[]);
+    }
+
     /// Removes all pages
-    pub fn clear(&self) {
+    fn clear(&self, lazy: bool) {
         self.scroll_animation().pause();
+        self.step_animation().pause();
 
         for page in self.imp().pages.borrow().iter() {
             page.unparent();
@@ -370,7 +552,9 @@ impl LpSlidingView {
         self.set_position(0.);
         self.imp().position_shift.set(0.);
 
-        self.set_current_page(None);
+        if !lazy {
+            self.set_current_page(None);
+        }
     }
 
     /// Gives all pages with hash access via their path
@@ -394,20 +578,67 @@ impl LpSlidingView {
             .cloned()
     }
 
-    /// Move to specified page with animation
-    pub fn scroll_to(&self, page: &LpImagePage) {
-        self.scroll_to_velocity(page, 0.);
+    /// Move to specified page without animation
+    pub fn instant_to(&self, page: &LpImagePage) {
+        if let Some(index) = self.index_of(page) {
+            self.step_animation().pause();
+            self.scroll_animation().pause();
+
+            self.set_position(index as f64 - self.position_shift());
+
+            self.set_current_page(Some(page));
+            self.emit_target_page_reached();
+        } else {
+            log::error!("Page not in LpSlidingView {}", page.file().uri());
+        }
     }
 
-    pub fn scroll_to_velocity(&self, page: &LpImagePage, initial_velocity: f64) {
+    /// Move to specified page with animation
+    pub fn animate_to(&self, page: &LpImagePage) {
+        if let Some(index) = self.index_of(page) {
+            if self
+                .current_index()
+                .map_or(false, |x| (x as i64 - index as i64).abs() == 1)
+            {
+                self.step_to(page);
+            } else {
+                self.scroll_to(page, 0.);
+            }
+        } else {
+            log::error!("Page not in LpSlidingView {}", page.file().uri());
+        }
+    }
+
+    /// Move to image with cards animation
+    fn step_to(&self, page: &LpImagePage) {
+        let animation = self.step_animation();
+        if let Some(prev_page) = self.current_page() {
+            if let (Some(prev_index), Some(new_index)) =
+                (self.index_of(&prev_page), self.index_of(page))
+            {
+                let backward = new_index < prev_index;
+
+                *self.imp().position_tracking.borrow_mut() =
+                    PositionTracking::StackedCards(StackedCards::new(prev_page, backward));
+                // Set current page here as well, to make sure it's set when animation completes
+                // for disabled animations
+                self.set_current_page(Some(page));
+                animation.play();
+            }
+        }
+        self.set_current_page(Some(page));
+    }
+
+    /// Move to image with scroll animation
+    fn scroll_to(&self, page: &LpImagePage, initial_velocity: f64) {
         if let Some(index) = self.index_of(page) {
             let animation = self.scroll_animation();
 
             animation.set_value_from(self.position());
             animation.set_value_to(index as f64 - self.position_shift());
             animation.set_initial_velocity(initial_velocity);
-            animation.play();
             self.set_current_page(Some(page));
+            animation.play();
         } else {
             log::error!("Page not in LpSlidingView {}", page.file().uri());
         }
@@ -419,11 +650,11 @@ impl LpSlidingView {
     /// the current page if last remaining.
     pub fn scroll_to_neighbor(&self) {
         if let Some(next_page) = self.next_page() {
-            self.scroll_to(&next_page);
+            self.step_to(&next_page);
         } else if let Some(prev_page) = self.prev_page() {
-            self.scroll_to(&prev_page);
+            self.step_to(&prev_page);
         } else if let Some(current_page) = self.current_page() {
-            self.remove(&current_page);
+            self.remove(&current_page, false);
         }
     }
 
@@ -457,7 +688,13 @@ impl LpSlidingView {
     }
 
     fn position(&self) -> f64 {
-        self.imp().position.get()
+        match &*self.imp().position_tracking.borrow() {
+            PositionTracking::Position(position) => *position,
+            PositionTracking::StackedCards(_) => {
+                log::error!("Using SlidindView position while in StackedCards animation");
+                0.
+            }
+        }
     }
 
     fn position_shift(&self) -> f64 {
@@ -466,8 +703,8 @@ impl LpSlidingView {
 
     /// Sets position shift to correct page insert or removal
     ///
-    /// Takes the old position of the current page and calculates and sets the shift
-    /// to keep the current page optically at the same position.
+    /// Takes the old position of the current page and calculates and sets the
+    /// shift to keep the current page optically at the same position.
     fn shift_position(&self, old_index: Option<usize>) {
         if let (Some(old_index), Some(new_index)) = (old_index, self.current_index()) {
             let shift = new_index as f64 - old_index as f64;
@@ -476,14 +713,13 @@ impl LpSlidingView {
     }
 
     fn set_position(&self, position: f64) {
-        self.imp().position.set(position);
+        self.imp()
+            .position_tracking
+            .borrow_mut()
+            .set_position(position);
         self.queue_allocate();
     }
 
-    pub fn current_page(&self) -> Option<LpImagePage> {
-        self.imp().current_page.borrow().clone()
-    }
-
     fn set_current_page(&self, page: Option<&LpImagePage>) {
         self.imp().current_page.replace(page.cloned());
         self.notify("current-page");
@@ -499,9 +735,11 @@ impl LpSlidingView {
 
     fn scroll_animation(&self) -> &adw::SpringAnimation {
         self.imp().scroll_animation.get_or_init(|| {
-            let target = adw::CallbackAnimationTarget::new(
-                glib::clone!(@weak self as obj => move |position| obj.set_position(position)),
-            );
+            let target = adw::CallbackAnimationTarget::new(glib::clone!(
+                #[weak(rename_to = obj)]
+                self,
+                move |position| obj.set_position(position)
+            ));
 
             let animation = adw::SpringAnimation::builder()
                 .spring_params(&adw::SpringParams::new(
@@ -513,11 +751,96 @@ impl LpSlidingView {
                 .target(&target)
                 .build();
 
-            animation.connect_done(glib::clone!(@weak self as obj => move |_| {
-                obj.emit_by_name("target-page-reached", &[])
-            }));
+            animation.connect_done(glib::clone!(
+                #[weak(rename_to = obj)]
+                self,
+                move |_| obj.emit_target_page_reached()
+            ));
+
+            animation
+        })
+    }
+
+    fn step_animation(&self) -> &adw::TimedAnimation {
+        self.imp().step_animation.get_or_init(|| {
+            let target: adw::CallbackAnimationTarget =
+                adw::CallbackAnimationTarget::new(glib::clone!(
+                    #[weak(rename_to = obj)]
+                    self,
+                    move |progress| {
+                        obj.imp()
+                            .position_tracking
+                            .borrow_mut()
+                            .set_progress(progress);
+                        obj.queue_allocate();
+                    }
+                ));
+
+            let animation = adw::TimedAnimation::builder()
+                .duration(STEP_DURATION)
+                .value_from(0.)
+                .value_to(1.)
+                .widget(self)
+                .target(&target)
+                .build();
+
+            animation.connect_done(glib::clone!(
+                #[weak(rename_to = obj)]
+                self,
+                move |_| {
+                    if let Some(current_index) = obj.current_index() {
+                        let imp = obj.imp();
+                        imp.position_tracking.borrow_mut().set_progress(1.);
+                        imp.position_tracking
+                            .borrow_mut()
+                            .set_position(current_index as f64 - obj.position_shift());
+                        obj.queue_allocate();
+                    } else {
+                        error!("No current page at end of animation");
+                    }
+                    obj.emit_target_page_reached();
+                }
+            ));
 
             animation
         })
     }
 }
+
+/// See [`LpSlidingView::editor`](LpSlidingView::editor)
+pub struct SlidingViewEditor {
+    sliding_view: LpSlidingView,
+}
+
+impl SlidingViewEditor {
+    fn new(sliding_view: LpSlidingView) -> Self {
+        Self { sliding_view }
+    }
+
+    /// Removes the page giving notifications later
+    pub fn remove_lazy(&self, page: &LpImagePage) {
+        self.remove(page, true)
+    }
+
+    /// Removes all pages giving notifications later
+    pub fn clear_lazy(&self) {
+        self.clear(true)
+    }
+}
+
+impl std::ops::Deref for SlidingViewEditor {
+    type Target = LpSlidingView;
+
+    fn deref(&self) -> &LpSlidingView {
+        &self.sliding_view
+    }
+}
+
+impl Drop for SlidingViewEditor {
+    fn drop(&mut self) {
+        // Trigger everything that hasn't been done by lazy updates
+        if self.is_empty() {
+            self.clear(false);
+        }
+    }
+}
