@@ -1,13 +1,14 @@
-// Copyright (c) 2020-2023 Christopher Davis
+// Copyright (c) 2020-2024 Christopher Davis
 // Copyright (c) 2022-2024 Sophie Herold
 // Copyright (c) 2022 Elton A Rodrigues
 // Copyright (c) 2022 Maximiliano Sandoval R
 // Copyright (c) 2023 Matteo Nardi
-// Copyright (c) 2023 Julian Hofer
 // Copyright (c) 2023 FineFindus
 // Copyright (c) 2023 qwel
 // Copyright (c) 2023 Huan Thieu Nguyen
-// Copyright (c) 2023 Sabri Ünal
+// Copyright (c) 2024 Fina Wilke
+// Copyright (c) 2024 DaPigGuy
+// Copyright (c) 2024 James Frost
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -24,22 +25,25 @@
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
+mod actions;
 mod controls;
 
 use std::cell::{Cell, OnceCell, RefCell};
+use std::marker::PhantomData;
 use std::path::{Path, PathBuf};
 
+use actions::*;
 use adw::prelude::*;
 use adw::subclass::prelude::*;
-use glib::clone;
+use glib::{clone, Properties};
 use gtk::{CompositeTemplate, Widget};
 
 use crate::application::LpApplication;
 use crate::config;
 use crate::deps::*;
 use crate::util::gettext::*;
-use crate::util::{Direction, Position};
-use crate::widgets::{LpDragOverlay, LpImage, LpImageView, LpPropertiesView};
+use crate::util::Direction;
+use crate::widgets::{LpDragOverlay, LpImage, LpImageView, LpPropertiesView, LpWindowContent};
 
 /// Show window after X milliseconds even if image dimensions are not known yet
 const SHOW_WINDOW_AFTER: u64 = 2000;
@@ -52,6 +56,7 @@ const HIDE_CONTROLS_ANIMATION_DURATION: u32 = 1000;
 const HIDE_CONTROLS_IDLE_TIMEOUT: u64 = 3000;
 
 mod imp {
+
     use gio::glib::VariantTy;
 
     use super::*;
@@ -70,14 +75,15 @@ mod imp {
     // If some member fields did not implement default,
     // we'd need to have a `new()` function in the
     // `impl ObjectSubclass for $TYPE` section.
-    #[derive(Default, Debug, CompositeTemplate)]
+    #[derive(Default, Debug, Properties, CompositeTemplate)]
+    #[properties(wrapper_type = super::LpWindow)]
     #[template(file = "window.ui")]
     pub struct LpWindow {
         // Template children are used with the
         // TemplateChild<T> wrapper, where T is the
         // object type of the template child.
         #[template_child]
-        pub(super) toolbar_view: TemplateChild<adw::ToolbarView>,
+        pub(super) window_content: TemplateChild<LpWindowContent>,
         #[template_child]
         pub(super) headerbar: TemplateChild<adw::HeaderBar>,
         #[template_child]
@@ -107,6 +113,12 @@ mod imp {
         #[template_child]
         pub(super) backward_click_gesture: TemplateChild<gtk::GestureClick>,
 
+        #[property(get = Self::is_showing_image)]
+        pub(super) is_showing_image: PhantomData<bool>,
+        /// Set via binding to HeaderBar
+        #[property(get, set)]
+        headerbar_opacity: Cell<f64>,
+
         /// Motion controller for complete window
         pub(super) motion_controller: gtk::EventControllerMotion,
         pub(super) pointer_position: Cell<(f64, f64)>,
@@ -137,148 +149,16 @@ mod imp {
                 }
             });
 
-            // Set up actions
-            klass.install_action("win.toggle-fullscreen", None, move |win, _, _| {
-                win.toggle_fullscreen(!win.is_fullscreened());
-            });
-
-            klass.install_action("win.next", None, move |win, _, _| {
-                win.imp().image_view.navigate(Direction::Forward, true);
-            });
-
-            klass.install_action("win.previous", None, move |win, _, _| {
-                win.imp().image_view.navigate(Direction::Back, true);
-            });
-
-            klass.install_action("win.image-right", None, move |win, _, _| {
-                if win.direction() == gtk::TextDirection::Rtl {
-                    win.imp().image_view.navigate(Direction::Back, true);
-                } else {
-                    win.imp().image_view.navigate(Direction::Forward, true);
-                }
-            });
-
-            klass.install_action("win.image-right-instant", None, move |win, _, _| {
-                if win.direction() == gtk::TextDirection::Rtl {
-                    win.imp().image_view.navigate(Direction::Back, false);
-                } else {
-                    win.imp().image_view.navigate(Direction::Forward, false);
-                }
-            });
-
-            klass.install_action("win.image-left", None, move |win, _, _| {
-                if win.direction() == gtk::TextDirection::Rtl {
-                    win.imp().image_view.navigate(Direction::Forward, true);
-                } else {
-                    win.imp().image_view.navigate(Direction::Back, true);
-                }
-            });
-
-            klass.install_action("win.image-left-instant", None, move |win, _, _| {
-                if win.direction() == gtk::TextDirection::Rtl {
-                    win.imp().image_view.navigate(Direction::Forward, false);
-                } else {
-                    win.imp().image_view.navigate(Direction::Back, false);
-                }
-            });
-
-            klass.install_action("win.first", None, move |win, _, _| {
-                win.imp().image_view.jump(Position::First);
-            });
-
-            klass.install_action("win.last", None, move |win, _, _| {
-                win.imp().image_view.jump(Position::Last);
-            });
-
-            klass.install_action("win.zoom-out-cursor", None, move |win, _, _| {
-                win.zoom_out_cursor();
-            });
-
-            klass.install_action("win.zoom-out-center", None, move |win, _, _| {
-                win.zoom_out_center();
-            });
-
-            klass.install_action("win.zoom-in-cursor", None, move |win, _, _| {
-                win.zoom_in_cursor();
-            });
-
-            klass.install_action("win.zoom-in-center", None, move |win, _, _| {
-                win.zoom_in_center();
-            });
-
-            klass.install_action(
-                "win.zoom-to-exact",
-                Some(VariantTy::DOUBLE),
-                move |win, _, level| {
-                    win.zoom_to_exact(level.unwrap().get().unwrap());
-                },
+            klass.add_binding_action(
+                gdk::Key::question,
+                gdk::ModifierType::CONTROL_MASK,
+                "win.show-help-overlay",
             );
 
-            klass.install_action("win.zoom-best-fit", None, move |win, _, _| {
-                win.zoom_best_fit();
-            });
+            // Set up actions
 
-            klass.install_action("win.pan-up", None, move |win, _, _| {
-                win.pan(&gtk::PanDirection::Up);
-            });
-
-            klass.install_action("win.pan-down", None, move |win, _, _| {
-                win.pan(&gtk::PanDirection::Down);
-            });
-
-            klass.install_action("win.pan-left", None, move |win, _, _| {
-                win.pan(&gtk::PanDirection::Left);
-            });
-
-            klass.install_action("win.pan-right", None, move |win, _, _| {
-                win.pan(&gtk::PanDirection::Right);
-            });
-
-            klass.install_action("win.leave-fullscreen", None, move |win, _, _| {
-                win.toggle_fullscreen(false);
-            });
-
-            klass.install_action("win.toggle-properties", None, move |win, _, _| {
-                win.imp()
-                    .properties_button
-                    .set_active(!win.imp().properties_button.is_active());
-            });
-
-            klass.install_action_async("win.open", None, |win, _, _| async move {
-                win.pick_file().await;
-            });
-
-            klass.install_action_async("win.open-with", None, |win, _, _| async move {
-                win.open_with().await;
-            });
-
-            klass.install_action("win.rotate_cw", None, |win, _, _| {
-                win.rotate_image(90.0);
-            });
-
-            klass.install_action("win.rotate_ccw", None, |win, _, _| {
-                win.rotate_image(-90.0);
-            });
-
-            klass.install_action_async("win.set-background", None, |win, _, _| async move {
-                win.set_background().await;
-            });
-
-            klass.install_action("win.print", None, move |win, _, _| {
-                win.print();
-            });
-
-            klass.install_action("win.copy-image", None, move |win, _, _| {
-                win.copy_image();
-            });
-
-            klass.install_action_async("win.trash", None, |win, _, _| async move {
-                win.trash().await;
-            });
-
-            klass.install_action_async("win.delete", None, |win, _, _| async move {
-                win.delete().await;
-            });
+            ActionPartGlobal::init_actions_and_bindings(klass);
+            Action::init_actions_and_bindings(klass);
 
             klass.install_action(
                 "win.show-toast",
@@ -296,24 +176,34 @@ mod imp {
         }
     }
 
+    #[glib::derived_properties]
     impl ObjectImpl for LpWindow {
         fn constructed(&self) {
             self.parent_constructed();
 
             let obj = self.obj();
 
-            self.forward_click_gesture
-                .connect_pressed(clone!(@weak obj => move |_,_,_,_| {
-                    obj.image_view().navigate(Direction::Forward, true);
-                }));
-            self.backward_click_gesture
-                .connect_pressed(clone!(@weak obj => move |_,_,_,_| {
-                    obj.image_view().navigate(Direction::Back, true);
-                }));
-            self.properties_button
-                .connect_toggled(clone!(@weak obj => move |_| {
+            self.forward_click_gesture.connect_pressed(clone!(
+                #[weak]
+                obj,
+                move |_, _, _, _| {
+                    obj.image_view().navigate(Direction::Forward, false);
+                }
+            ));
+            self.backward_click_gesture.connect_pressed(clone!(
+                #[weak]
+                obj,
+                move |_, _, _, _| {
+                    obj.image_view().navigate(Direction::Back, false);
+                }
+            ));
+            self.properties_button.connect_toggled(clone!(
+                #[weak]
+                obj,
+                move |_| {
                     obj.on_properties_button_toggled();
-                }));
+                }
+            ));
 
             if config::PROFILE == ".Devel" {
                 obj.add_css_class("devel");
@@ -325,59 +215,68 @@ mod imp {
 
             glib::timeout_add_local_once(
                 std::time::Duration::from_millis(SHOW_WINDOW_AFTER),
-                glib::clone!(@weak obj => move || if !obj.is_visible() { obj.present() }),
+                glib::clone!(
+                    #[weak]
+                    obj,
+                    move || if !obj.is_visible() {
+                        obj.present()
+                    }
+                ),
             );
 
             obj.on_current_page_changed();
             obj.on_fullscreen_changed();
 
-            self.image_view.connect_current_page_notify(
-                glib::clone!(@weak obj => move |_| obj.on_current_page_changed()),
-            );
+            self.image_view.connect_current_page_notify(glib::clone!(
+                #[weak]
+                obj,
+                move |_| obj.on_current_page_changed()
+            ));
 
-            obj.connect_fullscreened_notify(
-                glib::clone!(@weak obj => move |_| obj.on_fullscreen_changed()),
-            );
+            obj.connect_fullscreened_notify(glib::clone!(
+                #[weak]
+                obj,
+                move |_| obj.on_fullscreen_changed()
+            ));
 
             obj.connect_map(|win| {
                 win.resize_default();
             });
 
-            self.properties_button.connect_active_notify(
-                glib::clone!(@weak obj => move |_| obj.update_headerbar_style()),
-            );
-
             let gesture_click = gtk::GestureClick::new();
-            gesture_click
-                .connect_pressed(glib::clone!(@weak obj => move |_, _, _, _| obj.on_click()));
+            gesture_click.connect_pressed(glib::clone!(
+                #[weak]
+                obj,
+                move |_, _, _, _| obj.on_click()
+            ));
             obj.add_controller(gesture_click);
 
-            self.motion_controller
-                .connect_motion(glib::clone!(@weak obj => move |_, x, y| obj.on_motion((x,y))));
-            self.motion_controller
-                .connect_enter(glib::clone!(@weak obj => move |_, x, y| obj.on_motion((x,y))));
+            self.motion_controller.connect_motion(glib::clone!(
+                #[weak]
+                obj,
+                move |_, x, y| obj.on_motion((x, y))
+            ));
+            self.motion_controller.connect_enter(glib::clone!(
+                #[weak]
+                obj,
+                move |_, x, y| obj.on_motion((x, y))
+            ));
             obj.add_controller(self.motion_controller.clone());
 
             let current_image_signals = self.image_view.current_image_signals();
-            // clone! is a macro from glib-rs that allows
-            // you to easily handle references in callbacks
-            // without refcycles or leaks.
-            //
-            // When you don't want the callback to keep the
-            // Object alive, pass as @weak. Otherwise, pass
-            // as @strong. Most of the time you will want
-            // to use @weak.
-            current_image_signals.connect_bind_local(glib::clone!(@weak obj => move |_, _| {
-                obj.on_zoom_status_changed()
-            }));
 
             current_image_signals.connect_local(
                 "metadata-changed",
                 true,
-                glib::clone!(@weak obj => @default-return None, move |_| {
-                    obj.update_title();
-                    None
-                }),
+                glib::clone!(
+                    #[weak]
+                    obj,
+                    #[upgrade_or_default]
+                    move |_| {
+                        obj.update_title();
+                        None
+                    }
+                ),
             );
 
             current_image_signals.connect_closure(
@@ -386,57 +285,89 @@ mod imp {
                 // `closure_local!` is similar to `clone`, but you use `@watch` instead of clone.
                 // `@watch` means that this signal will be disconnected when the watched object
                 // is dropped.
-                glib::closure_local!(@watch obj => move |_: &LpImage, _: &glib::ParamSpec| {
-                    obj.on_zoom_status_changed();
-                }),
+                glib::closure_local!(
+                    #[watch]
+                    obj,
+                    move |_: &LpImage, _: &glib::ParamSpec| {
+                        obj.update_image_actions_status();
+                    }
+                ),
             );
 
             current_image_signals.connect_closure(
                 "notify::is-max-zoom",
                 false,
-                glib::closure_local!(@watch obj => move |_: &LpImage, _: &glib::ParamSpec| {
-                    obj.on_zoom_status_changed();
-                }),
+                glib::closure_local!(
+                    #[watch]
+                    obj,
+                    move |_: &LpImage, _: &glib::ParamSpec| {
+                        obj.update_image_actions_status();
+                    }
+                ),
             );
 
             current_image_signals.connect_closure(
                 "notify::image-size-available",
                 false,
-                glib::closure_local!(@watch obj => move |_: &LpImage, _: &glib::ParamSpec| {
-                    obj.image_size_available();
-                }),
+                glib::closure_local!(
+                    #[watch]
+                    obj,
+                    move |_: &LpImage, _: &glib::ParamSpec| {
+                        obj.image_size_available();
+                    }
+                ),
+            );
+
+            current_image_signals.connect_closure(
+                "notify::is-loaded",
+                false,
+                glib::closure_local!(
+                    #[watch]
+                    obj,
+                    move |_: &LpImage, _: &glib::ParamSpec| {
+                        obj.update_image_actions_status();
+                    }
+                ),
             );
 
             current_image_signals.connect_closure(
                 "notify::error",
                 false,
-                glib::closure_local!(@watch obj => move |_: &LpImage, _: &glib::ParamSpec| {
-                    obj.image_error();
-                }),
-            );
-
-            self.toolbar_view.connect_top_bar_style_notify(
-                glib::clone!(@weak obj => move |_| obj.on_top_bar_style_notify()),
+                glib::closure_local!(
+                    #[watch]
+                    obj,
+                    move |_: &LpImage, _: &glib::ParamSpec| {
+                        obj.image_error();
+                        obj.update_image_actions_status();
+                    }
+                ),
             );
 
             // action win.previous status
-            self.image_view.connect_is_previous_available_notify(
-                glib::clone!(@weak obj => move |_| {
-                    obj.action_set_enabled(
-                        "win.previous",
-                        obj.imp().image_view.is_previous_available(),
-                    );
-                }),
-            );
+            self.image_view
+                .connect_is_previous_available_notify(glib::clone!(
+                    #[weak]
+                    obj,
+                    move |_| {
+                        obj.action_set_enabled(
+                            &Action::Previous,
+                            obj.imp().image_view.is_previous_available(),
+                        );
+                    }
+                ));
 
             // action win.next status
             self.image_view
-                .connect_is_next_available_notify(glib::clone!(@weak obj => move |_| {
-                    obj.action_set_enabled(
-                        "win.next",
-                        obj.imp().image_view.is_next_available(),
-                    );
-                }));
+                .connect_is_next_available_notify(glib::clone!(
+                    #[weak]
+                    obj,
+                    move |_| {
+                        obj.action_set_enabled(
+                            &Action::Next,
+                            obj.imp().image_view.is_next_available(),
+                        );
+                    }
+                ));
 
             // Make widgets visible when the focus moves
             obj.connect_move_focus(|obj, _| {
@@ -444,36 +375,50 @@ mod imp {
                 obj.schedule_hide_controls();
             });
 
+            // Activate global shortcuts only if no dialog is open
+            obj.connect_visible_dialog_notify(|obj| obj.update_accel_status());
+            obj.connect_is_active_notify(|obj| obj.update_accel_status());
+
             self.status_page
                 .set_icon_name(Some(&format!("{}-symbolic", config::APP_ID)));
 
             self.drop_target.set_types(&[gdk::FileList::static_type()]);
 
-            self.drop_target.connect_accept(
-                clone!(@weak obj => @default-return false, move |_drop_target, drop| {
+            self.drop_target.connect_accept(clone!(
+                #[weak]
+                obj,
+                #[upgrade_or]
+                false,
+                move |_drop_target, drop| {
                     // Only accept drops from external sources or different windows
-                    let different_source = drop.drag().is_none() || drop.drag() != obj.image_view().drag_source().drag();
+                    let different_source = drop.drag().is_none()
+                        || drop.drag() != obj.image_view().drag_source().drag();
                     // We have to do this manually since we are overwriting the default handler
                     let correct_format = drop.formats().contains_type(gdk::FileList::static_type());
 
                     different_source && correct_format
-                }),
-            );
+                }
+            ));
 
             // For callbacks, you will want to reference the GTK docs on
             // the relevant signal to see which parameters you need.
             // In this case, we need only need the GValue,
             // so we name it `value` then use `_` for the other spots.
-            self.drop_target.connect_drop(
-                clone!(@weak obj => @default-return false, move |_, value, _, _| {
-
-                    // Here we use a GValue, which is a dynamic object that can hold different types,
-                    // e.g. strings, numbers, or in this case objects. In order to get the GdkFileList
-                    // from the GValue, we need to use the `get()` method.
+            self.drop_target.connect_drop(clone!(
+                #[weak]
+                obj,
+                #[upgrade_or]
+                false,
+                move |_, value, _, _| {
+                    // Here we use a GValue, which is a dynamic object that can hold different
+                    // types, e.g. strings, numbers, or in this case objects. In
+                    // order to get the GdkFileList from the GValue, we need to
+                    // use the `get()` method.
                     //
-                    // We've added type annotations here, and written it as `let list: gdk::FileList = ...`,
-                    // but you might also see places where type arguments are used.
-                    // This line could have been written as `let list = value.get::<gdk::FileList>().unwrap()`.
+                    // We've added type annotations here, and written it as `let list: gdk::FileList
+                    // = ...`, but you might also see places where type
+                    // arguments are used. This line could have been written as
+                    // `let list = value.get::<gdk::FileList>().unwrap()`.
                     let files = match value.get::<gdk::FileList>() {
                         Ok(list) => list.files(),
                         Err(err) => {
@@ -493,8 +438,8 @@ mod imp {
                     obj.present();
 
                     true
-                }),
-            );
+                }
+            ));
         }
     }
 
@@ -502,6 +447,16 @@ mod imp {
     impl WindowImpl for LpWindow {}
     impl ApplicationWindowImpl for LpWindow {}
     impl AdwApplicationWindowImpl for LpWindow {}
+
+    impl LpWindow {
+        fn is_showing_image(&self) -> bool {
+            self.image_view.try_get().map_or(false, |image_view| {
+                image_view
+                    .current_page()
+                    .map_or(false, |page| page.image().error().is_none())
+            })
+        }
+    }
 }
 
 glib::wrapper! {
@@ -566,8 +521,8 @@ impl LpWindow {
 
         let filter_supported_formats = gtk::FileFilter::new();
         filter_supported_formats.set_name(Some(&gettext("Supported image formats")));
-        for mime_type in glycin::supported_mime_types().await {
-            filter_supported_formats.add_mime_type(&mime_type);
+        for mime_type in glycin::Loader::supported_mime_types().await {
+            filter_supported_formats.add_mime_type(mime_type.as_str());
         }
 
         let filter_all_files = gtk::FileFilter::new();
@@ -645,7 +600,7 @@ impl LpWindow {
 
     /// Returns true if some text in metadata is currently selected
     fn has_metadata_selected(&self) -> bool {
-        if let Some(focus_widget) = self.focus_widget() {
+        if let Some(focus_widget) = self.focus() {
             if focus_widget.is_ancestor(&*self.imp().properties_view) {
                 if let Ok(label) = focus_widget.downcast::<gtk::Label>() {
                     return label.selection_bounds().is_some();
@@ -689,21 +644,25 @@ impl LpWindow {
                     .button_label(gettext("Undo"))
                     .priority(adw::ToastPriority::High)
                     .build();
-                toast.connect_button_clicked(glib::clone!(@weak self as win => move |_| {
-                    let path = path.clone();
-                    glib::spawn_future_local(async move {
-                        win.image_view()
-                            .set_trash_restore(Some(gio::File::for_path(&path)));
-                        let result = crate::util::untrash(&path).await;
-                        if let Err(err) = result {
-                            log::error!("Failed to untrash {path:?}: {err}");
-                            win.show_toast(
-                                gettext("Failed to restore image from trash"),
-                                adw::ToastPriority::High,
-                            );
-                        }
-                    });
-                }));
+                toast.connect_button_clicked(glib::clone!(
+                    #[weak(rename_to = win)]
+                    self,
+                    move |_| {
+                        let path = path.clone();
+                        glib::spawn_future_local(async move {
+                            win.image_view()
+                                .set_trash_restore(Some(gio::File::for_path(&path)));
+                            let result = crate::util::untrash(&path).await;
+                            if let Err(err) = result {
+                                log::error!("Failed to untrash {path:?}: {err}");
+                                win.show_toast(
+                                    gettext("Failed to restore image from trash"),
+                                    adw::ToastPriority::High,
+                                );
+                            }
+                        });
+                    }
+                ));
                 self.imp().toast_overlay.add_toast(toast);
             }
             Err(err) => {
@@ -735,9 +694,7 @@ impl LpWindow {
     ///
     /// Fallback for when trash not available or explicit call with shortcut
     async fn delete_future(&self, path: &Path) {
-        let dialog = adw::MessageDialog::builder()
-            .modal(true)
-            .transient_for(self)
+        let dialog = adw::AlertDialog::builder()
             .heading(gettext("Permanently Delete Image?"))
             .body(gettext_f(
                 "After deleting the image “{}” it will be permanently lost.",
@@ -753,7 +710,7 @@ impl LpWindow {
         ]);
         dialog.set_response_appearance("delete", adw::ResponseAppearance::Destructive);
 
-        if "delete" == dialog.choose_future().await {
+        if "delete" == dialog.choose_future(self).await {
             let file = gio::File::for_path(path);
             let result = file.delete_future(glib::Priority::default()).await;
 
@@ -784,22 +741,59 @@ impl LpWindow {
         imp.toast_overlay.add_toast(toast);
     }
 
-    pub fn set_actions_enabled(&self, enabled: bool) {
-        const ACTIONS: &[&str] = &[
-            "win.open-with",
-            "win.set-background",
-            "win.toggle-fullscreen",
-            "win.print",
-            "win.rotate_cw",
-            "win.rotate_ccw",
-            "win.copy-image",
-            "win.zoom-best-fit",
-            "win.zoom-to-exact",
-            "win.toggle-properties",
+    /// Set status for actions that are related to the active image
+    fn update_image_actions_status(&self) {
+        // Actions that are enabled if the image is shown
+        const ACTIONS_SHOWN: &[Action] = &[
+            Action::SetBackground,
+            Action::Print,
+            Action::RotateCw,
+            Action::RotateCcw,
+            Action::ZoomBestFit,
+            Action::ZoomToExact1,
+            Action::ZoomToExact2,
         ];
 
-        for action in ACTIONS {
-            self.action_set_enabled(action, enabled);
+        let enabled_shown = self
+            .imp()
+            .image_view
+            .current_page()
+            .is_some_and(|x| x.image().is_loaded() && x.image().error().is_none());
+
+        for action in ACTIONS_SHOWN {
+            self.action_set_enabled(action, enabled_shown);
+        }
+
+        // Zoom specific actions
+
+        let can_zoom_out = self
+            .image_view()
+            .current_image()
+            .is_some_and(|image| !image.is_best_fit());
+        let can_zoom_in = self
+            .image_view()
+            .current_image()
+            .is_some_and(|image| !image.is_max_zoom());
+
+        self.action_set_enabled(&Action::ZoomOutCursor, can_zoom_out && enabled_shown);
+        self.action_set_enabled(&Action::ZoomOutCenter, can_zoom_out && enabled_shown);
+        self.action_set_enabled(&Action::ZoomInCursor, can_zoom_in && enabled_shown);
+        self.action_set_enabled(&Action::ZoomInCenter, can_zoom_in && enabled_shown);
+
+        // Actions that are available if there is an current image, even if it's not
+        // shown
+        const ACTIONS_CURRENT: &[Action] = &[
+            Action::OpenWith,
+            Action::CopyImage,
+            Action::ToggleProperties,
+            Action::Reload,
+            Action::ToggleFullscreen,
+        ];
+
+        let enabled_current = self.imp().image_view.current_page().is_some();
+
+        for action in ACTIONS_CURRENT {
+            self.action_set_enabled(action, enabled_current);
         }
     }
 
@@ -811,7 +805,7 @@ impl LpWindow {
         let current_page = imp.image_view.current_page();
 
         // HeaderBar style
-        self.update_headerbar_style();
+        self.notify_is_showing_image();
 
         // Window title
         self.update_title();
@@ -822,9 +816,9 @@ impl LpWindow {
 
         let has_image = current_page.is_some();
 
-        self.set_actions_enabled(has_image);
+        self.update_image_actions_status();
         self.action_set_enabled(
-            "win.trash",
+            &Action::Trash,
             imp.image_view
                 .current_file()
                 .is_some_and(|file| file.path().is_some()),
@@ -901,17 +895,42 @@ impl LpWindow {
     }
 
     pub fn resize_default(&self) {
+        let imp = self.imp();
+
         if self
             .image_view()
             .current_image()
             .is_some_and(|img| img.image_size_available())
         {
-            // this let's the window determine the default size from LpImage's natural size
-            self.set_default_size(-1, -1);
+            let shows_properties = imp.properties_button.is_active();
+
+            let (_, window_natural_width, _, _) = self.measure(gtk::Orientation::Horizontal, -1);
+            let (_, window_natural_height, _, _) = self.measure(gtk::Orientation::Vertical, -1);
+
+            // These have to be in sync with the "conditions" for the "overlay-properties"
+            // breakpoint
+            let min_width_for_overlay = adw::LengthUnit::Sp.to_px(590., None).ceil() as i32;
+            let min_height_for_overlay = adw::LengthUnit::Sp.to_px(550., None).ceil() as i32;
+
+            let (width, height) = if shows_properties
+                && window_natural_width < min_width_for_overlay
+                && window_natural_height < min_height_for_overlay
+            {
+                // Avoid overlaying bottom sheet being triggered for the image properties by
+                // using a window wide enough to allow for a sidebar
+                (min_width_for_overlay.saturating_add(1), -1)
+            } else {
+                // this lets the window determine the default size from LpImage's natural size
+                (-1, -1)
+            };
+
+            self.set_default_size(width, height);
         }
     }
 
     pub fn image_error(&self) {
+        self.notify_is_showing_image();
+
         if self.is_visible() {
             return;
         }
@@ -924,32 +943,12 @@ impl LpWindow {
         }
     }
 
-    fn on_zoom_status_changed(&self) {
-        let can_zoom_out = self
-            .image_view()
-            .current_image()
-            .map(|image| !image.is_best_fit())
-            .unwrap_or_default();
-        let can_zoom_in = self
-            .image_view()
-            .current_image()
-            .map(|image| !image.is_max_zoom())
-            .unwrap_or_default();
-
-        self.action_set_enabled("win.zoom-out-cursor", can_zoom_out);
-        self.action_set_enabled("win.zoom-out-center", can_zoom_out);
-        self.action_set_enabled("win.zoom-in-cursor", can_zoom_in);
-        self.action_set_enabled("win.zoom-in-center", can_zoom_in);
-    }
-
     fn on_fullscreen_changed(&self) {
         self.imp()
             .image_view
-            .on_fullscreen_changed(self.is_fullscreened());
+            .on_fullscreen_changed(self.is_fullscreen());
 
-        self.update_headerbar_style();
-
-        if !self.is_fullscreened() {
+        if !self.is_fullscreen() {
             self.set_cursor(None);
             self.show_controls();
         }
@@ -957,38 +956,33 @@ impl LpWindow {
     }
 
     fn is_headerbar_flat(&self) -> bool {
-        matches!(
-            self.imp().toolbar_view.top_bar_style(),
-            adw::ToolbarStyle::Flat
-        )
-    }
-
-    fn on_top_bar_style_notify(&self) {
-        if self.is_headerbar_flat() {
-            // Bring headerbar opacity in sync with controls
-            self.headerbar().set_opacity(self.controls_opacity());
-        } else {
-            self.headerbar().set_opacity(1.);
-        }
+        self.imp().window_content.is_headerbar_flat()
     }
 
     fn is_content_extended_to_top(&self) -> bool {
-        self.is_fullscreened() && !self.imp().properties_button.is_active()
+        self.imp().window_content.is_content_extended_to_top()
     }
 
-    fn update_headerbar_style(&self) {
-        let imp = self.imp();
-
-        let extend = self.is_content_extended_to_top();
-        imp.toolbar_view.set_extend_content_to_top_edge(extend);
-
-        // Flat headerbar for empty state and fullscreen
-        let toolbar_style = if imp.image_view.current_image().is_none() || extend {
-            adw::ToolbarStyle::Flat
-        } else {
-            // Use the border variant of raised to avoid shadows over images
-            adw::ToolbarStyle::RaisedBorder
+    fn update_accel_status(&self) {
+        let Some(application) = self.application() else {
+            log::error!("No application for window found");
+            return;
         };
-        imp.toolbar_view.set_top_bar_style(toolbar_style);
+
+        // Only change status if active window
+        if self.is_active() {
+            if self.visible_dialog().is_some() {
+                // If AdwDialog is visible, remove global accels that are for the main window
+                ActionPartGlobal::remove_accels(&application);
+            } else {
+                // Add accels if viewing main window
+                ActionPartGlobal::add_accels(&application);
+            }
+        }
+    }
+
+    async fn show_about(&self) {
+        let about = crate::about::dialog().await;
+        about.present(Some(self));
     }
 }
